class Hex(object):
    def __init__(self, *args):
        self.bytes = []
        for arg in args:
            if isinstance(arg, (int,long)):
                self.bytes.append(self.create_from_int(arg).bytes)
            elif isinstance(arg, Hex):
                self.bytes.append(arg.bytes)
            else:
                raise TypeError,"cannot convert to Hex object: '%s'"%arg
        self.bytes = sum(self.bytes,[])

    @classmethod
    def create_from_int(cls, i):
        assert isinstance(i,(int,long)), "type of i is %s"%type(i)
        h = Hex()
        h.bytes = []
        v = i
        if v == 0:
            h.bytes = [0]
        else:
            while v:
                v,r = divmod(v,0x100)
                h.bytes.insert(0,r)
        return h


    def __concat__(self, other):
        if isinstance(other, Hex):
            h = Hex()
            h.bytes = self.bytes+other.bytes
            return h
        else:
            h = Hex()
            h.bytes = self.bytes+Hex(other).bytes
            return h

    def __rconcat__(self, other):
        return Hex(other).__concat__(self)

    def __repr__(self):
        return " ".join(["0x%02X"%item for item in self.bytes])

'''
import binascii

class HexFormat:
    F_blank   = 0  # "00 01 02..."
    F_0x      = 1  # "0x00 0x01 0x02..."
    F_compact = 2  # "000102..."
    F_ascii   = 3  # ASCII text

class Hex(object):
    def __init__(self, *args):
        self.format = HexFormat.F_0x
        if len(args) == 0:
            self.bytes = ()
        elif len(args) == 1 and isinstance(args[0],Hex):  # copy construction
            self.bytes  = args[0].bytes
            self.format = args[0].format
        else:
            self.bytes = []
            for arg in args:
                if isinstance(arg, (int,long)):
                    self.bytes.append(self.create_from_int(arg).bytes)
                elif isinstance(arg, basestring):
                    self.bytes.append(self.create_from_str(arg).bytes)
                elif isinstance(arg, Hex):
                    self.bytes.append(arg.bytes)
                elif hasattr(arg, "bytes"):
                    try:
                        arg.bytes()
                    except TypeError:
                        if isinstance(arg.bytes, (list, tuple)):
                            self.bytes.append(tuple(arg.bytes))
                        else:
                            self.bytes.append(Hex(arg.bytes).bytes)
                else:
                    raise TypeError,"cannot convert object '%s' of type '%s' into Hex"%(arg, type(arg))
            self.bytes = sum(self.bytes,())


    @classmethod
    def create_from_hexstr(cls, s):
        s = s.replace("0x","")
        s = s.replace(" ","")
        h = Hex()
        h.bytes = tuple([int(s[i:i+2],16) for i in range(len(s))[::2]])
        return h

    @classmethod
    def create_from_int(cls, i):
        assert isinstance(i,(int,long)), "type of i is %s"%type(i)
        h = Hex()
        h.bytes = []
        v = i
        if v == 0:
            h.bytes = (0,)
        else:
            while v:
                v,r = divmod(v,0x100)
                h.bytes.insert(0,r)
        h.bytes = tuple(h.bytes)
        return h

    @classmethod
    def create_from_decstr(cls, ds):
        return self.create_from_int(int(ds))

    def __concat__(self, other):
        if isinstance(other, Hex):
            h = Hex()
            h.bytes = self.bytes+other.bytes
            return h
        else:
            h = Hex()
            h.bytes = self.bytes+Hex(other).bytes
            return h

    def __rconcat__(self, other):
        return Hex(other).__concat__(self)


    def __eq__(self, other):
        if hasattr(other, "bytes"):
            return self.bytes == other.bytes
        try:
            return self.bytes == Hex(other).bytes
        except TypeError:
            return False

    def __neq__(self, other):
        return not self.__eq__(other)

    def _get_num(self):
        if self.bytes == ():
            return 0
        sum = 0
        for i,item in enumerate(self.bytes[::-1]):
            sum+= item<<(8*i)
        return sum

    num = property(_get_num)

    def __lshift__(self, k):
        a,r = divmod(k,8)
        bytes = []
        v = 0
        for byte in self.bytes[::-1]:
            v,r2 = divmod(v+byte<<r,0x100)
            bytes.insert(0,r2)
        if v:
            bytes.insert(0,v)
        h = Hex()
        h.bytes = tuple(bytes+[0]*a)
        return h

    def __rshift__(self, k):
        a,r = divmod(k,8)
        bytes = []
        r1 = 0
        for byte in self.bytes[:len(self.bytes)-a]:
            v,r2 = divmod((byte<<8)>>r,0x100)
            bytes.append(v+r1)
            r1 = r2
        h = Hex()
        h.bytes = tuple(bytes)
        return h

    def binary(self):
        form = self.format
        self.format = HexFormat.F_compact
        s = self.__repr__()
        self.format = form
        return binascii.a2b_hex(s)

    def ascii(self):
        return "".join([chr(byte) for byte in self.bytes])

    def digsits(self):
        """
        The digits method returns a generator that iterates over the digits
        of this Hex object.
        """
        def digit_generator():
            "digit iterator for Hex object."
            for item in self.bytes:
                yield item>>4
                yield item&0x0F
        return digit_generator()

    def swap(self):
        h = Hex()
        h.bytes = []
        for byte in self.bytes:
            high = byte>>4
            low  = byte&0x0F
            h.bytes.append(high+low<<4)
        h.bytes = tuple(h.bytes)
        return h

    def __getitem__(self,i):
        return Hex(self.bytes[i])

    def __getslice__(self,i,j):
        return Hex(*self.bytes[i:j])

    def __nonzero__(self):
        return self != 0

    def __add__(self, other):
        if isinstance(other,(int,long)):
            return Hex(other+self.num)
        else:
            return Hex(other.num + self.num)

    def __radd__(self,other):
        if isinstance(other,(int,long)):
            return Hex.__add__(self, other)
        raise TypeError,other

    def __rmul__(self,other):
        if isinstance(other,(int,long)):
            return Hex.__mul__(self,other)
        raise TypeError,other

    def __mul__(self,other):
        if isinstance(other,(int,long)):
            return Hex(other*self.num)
        else:
            return Hex(other.num*self.num)

    def __rxor__(self,other):
        if isinstance(other,(int,long)):
            return Hex.__xor__(self,other)
        raise TypeError,other

    def __xor__(self,other):
        if isinstance(other,(int,long)):
            return Hex(other^self.num)
        else:
            return Hex(other.num^self.num)

    def __rsub__(self,other):
        if isinstance(other,(int,long)):
            return Hex.__sub__(self,other)
        raise TypeError,other

    def __mod__(self, other):
        if isinstance(other,(int,long)):
            return Hex(self.num%other)
        else:
            return Hex(self.num%other.num)

    def __sub__(self,other):
        if isinstance(other,(int,long)):
            return Hex(self.num-other)
        else:
            return Hex(self.num-other.num)

    def __rdiv__(self,other):
        if isinstance(other,(int,long)):
            return Hex.__div__(self,other)
        raise TypeError,other

    def __div__(self,other):
        if isinstance(other,(int,long)):
            return Hex(self.num/other)
        else:
            return Hex(self.num/other.num)

    def __repr__(self):
        strrep = []
        if self.format == HexFormat.F_0x:
            for item in self.bytes:
                strrep.append("0x%02X"%item)
            return " ".join(strrep)
        elif self.format == HexFormat.F_blank:
            for item in self.bytes:
                strrep.append("%02X"%item)
            return " ".join(strrep)
        elif self.format == HexFormat.F_compact:
            for item in self.bytes:
                strrep.append("%02X"%item)
            return "".join(strrep)
        elif self.format == HexFormat.F_ascii:
            return self.ascii()
        raise AttributeError, "No type appropriate format type available"



'''
