#
# This test requires gallery to run
#

import __fiber__
import unittest
from test import test_support
import parser
from EasyExtend.csttools import*
from EasyExtend.eecompiler import EECompiler
import symbol

FiberTransformer= __fiber__.FiberTransformer
PyGrammar       = __fiber__.PyGrammar
unparse     = __fiber__.unparse
prepare_source  = __fiber__.prepare_source
macro           = __fiber__.macro

eec = EECompiler(__fiber__)

class TestMacro3(unittest.TestCase):
    def runTest(self):pass
    def setUp(self):
        source ="""
        def f():
            while 1:
                if A|B:
                    break
                else:
                    x-=1
        """
        self.source = prepare_source(source)
        self.source_node = parser.suite(self.source).tolist()
        _if_stmt = find_node(self.source_node, symbol.if_stmt)
        _test  = find_all(_if_stmt, symbol.test)
        _suite = find_all(_if_stmt, symbol.suite)
        self.sym_node = {}
        self.sym_node["suite"] = _suite
        self.sym_node["test"]  = _test
        self.sym_node["BLOCK"] = any_stmt(find_node(self.source_node, symbol.while_stmt))


    def test_cit1(self):
        source = """
        if 1:
            (cit <suite>[0])
        if 2:
            (cit <suite>[1])
        """
        cst = macro(source).expand(self.sym_node)
        print unparse(cst)
        #code = EasyExtend.eecompiler.eecompile_cst(cst)

    def test_cit2(self):
        source = """
        (cit 2+j)
        """
        cst = macro(source).expand(self.sym_node, locals={"j":9})
        print unparse(cst)
        #code = EasyExtend.eecompiler.eecompile_cst(cst)

    def test_cit3(self):
        source = """
        f(cit 2+j) + g(cit j*9)
        """
        cst = macro(source).expand(self.sym_node, locals={"j":9})
        print unparse(cst)
        #code = EasyExtend.eecompiler.eecompile_cst(cst)

    def test_option1(self):
        source = """
        if x:
            print
        optional i in range(cit len(<suite>)):
            elif (cit <test>[i]):
                (cit <suite>[i])
        """
        cst = macro(source).expand(self.sym_node)
        print unparse(cst)

    def test_option2(self):
        source = """
        optional <test>:
            "OK when appears"
        optional <X>:
            "NOK when appears"
        <BLOCK>
        """
        cst = macro(source).expand(self.sym_node)
        print unparse(cst)

    def test_star_test1(self):
        source = """
        \nf(*<test>)
        """
        cst = macro(source).expand(self.sym_node)
        print unparse(cst)

    def test_star_test2(self):
        source = """
        \nf(<*test>)
        """
        cst = macro(source).expand(self.sym_node)
        print unparse(cst)

class TestMacro4(unittest.TestCase):
    def runTest(self):pass
    def setUp(self):
        source ="""
        def f():
            while 1:
                if A-1:
                    break
                else:
                    x-=1
        """
        self.source = prepare_source(source)
        self.source_node = parser.suite(self.source).tolist()
        _if_stmt = find_node(self.source_node, symbol.if_stmt)
        _test  = find_all(_if_stmt, symbol.test)
        _suite = find_all(_if_stmt, symbol.suite)
        self.sym_node = {}
        self.sym_node["suite"] = _suite
        self.sym_node["test"]  = _test
        for key,val in self.sym_node.items():
            self.sym_node[key] = tuple(val)

        target = """
        if x == 0:
            print
        optional i,item in enumerate(listify(*<test>)):
            elif select(x, (cit i)) < 2:
                return (cit item) +1
        """
        self.target = prepare_source(target)
        self.target_node = eec.eeparse_suite(self.target)
        self._optional = find_node(self.target_node, __fiber__.symbol.optional_stmt)


    def test_create_suite_versions(self):
        ft = FiberTransformer(__fiber__, symbol_map = self.sym_node)
        res = ft.create_suite_versions(find_node(self._optional, symbol.suite), [{'i':0, 'item':7},{'i':1, 'item':"A|B"}])
        ft.terminate()
        for item in res:
            print unparse(item)



    def test_eval_target(self):
        # print "DEBUG-test_eval_target\n",self.target
        cst = macro(self.target).expand(self.sym_node)
        print unparse(cst)

class TestMacro5(unittest.TestCase):
    def runTest(self):pass
    def setUp(self):
        source ="""
        a = 0
        b = 1
        c = 2
        d = 3
        """
        self.source = prepare_source(source)
        self.source_node = parser.suite(self.source).tolist()
        _expr_stmt = find_all(self.source_node, symbol.expr_stmt)
        self.sym_node = {}
        self.sym_node["expr_stmt"] = _expr_stmt


    def test_pure_option(self):
        target = """
        optional item in <*expr_stmt>:
            print (cit unparse(item).strip())
            (cit item)
        """
        cst = macro(target).expand(self.sym_node)
        print unparse(cst)



def test_main():
    test_support.run_unittest(TestMacro3)
    test_support.run_unittest(TestMacro4)
    test_support.run_unittest(TestMacro5)

if __name__ == "__main__":
    test_main()

